/**************************************************************
 * connectionless client programming  
 * 
 * Function: iterative echo client 
 *
 * Reference: Richard Stevens, Unix Network Programming, 1998
 *
 * Created by: Hairong Qi
 **************************************************************/
#include <stdio.h>        /* for standard I/O */
#include <unistd.h>       /* for read() and write() */
#include <strings.h>      /* for bzero() */
#include <stdlib.h>       /* for atoi() */
#include <sys/socket.h>
#include <sys/types.h>    /* for accept() and connect() */
#include <sys/errno.h>    /* for perror() */
#include <netinet/in.h>   /* for htonl() */
#include <netdb.h>        /* for gethostbyname() */

#define BSIZE 256
#define USAGE "./iclientudp server portno\n"

/**
 ** output the error message and exit
 **/
void error(char *msg)
{
  perror(msg);
  exit(1);
}


/**
 ** the main function 
 **/
int main(int argc, char *argv[])
{
  int connfd;              /* socket file descriptor */
  int portno;
  struct hostent *server;
  char buffer[BSIZE];
  struct sockaddr_in serv_addr, cli_addr;
  int n, clilen;

  /* check the correctness of command-line inputs */
  if (argc < 3) 
    error(USAGE);

  /* create a socket */
  connfd = socket(PF_INET, SOCK_DGRAM, 0);  /* change from SOCK_STREAM to SOCK_DGRAM */
  if (connfd < 0) 
    error("ERROR opening socket");

  /* specify local address */
  bzero((char *) &serv_addr, sizeof(serv_addr));
  serv_addr.sin_family = AF_INET;
  server = gethostbyname(argv[1]);
  if (server == NULL) 
    error("ERROR, no such host\n");
  bcopy((char *)server->h_addr, (char *)&serv_addr.sin_addr.s_addr, server->h_length);
  portno = atoi(argv[2]);
  serv_addr.sin_port = htons(portno);

  /* start sending message */
  while (1) {
    printf("Please enter the message: ");
    bzero(buffer, BSIZE);
    fgets(buffer, BSIZE-1, stdin);

    n = sendto(connfd, buffer, strlen(buffer), 0, (struct sockaddr *)&serv_addr, 
	       sizeof(serv_addr));
    if (n < 0) 
      error("ERROR writing to socket");
    
    /* start receiving message */
    bzero(buffer, BSIZE);
    n = recvfrom(connfd, buffer, BSIZE-1, 0, (struct sockaddr *)&cli_addr,
		 &clilen);
    if (n < 0)
      error("ERROR reading from socket");
    printf("%s\n\n", buffer);
    fflush(stdout);
  }

  return 0; 
}

